@@ -1,4 +1,6 @@
-/* $Id: isakmp_quick.c,v 1.13.2.6 2005/06/04 21:50:58 manubsd Exp $ */
+/*	$NetBSD: isakmp_quick.c,v 1.11.4.1 2007/08/01 11:52:21 vanhu Exp $	*/
+
+/* Id: isakmp_quick.c,v 1.29 2006/08/22 18:17:17 manubsd Exp */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -51,13 +53,12 @@
 #  include <time.h>
 # endif
 #endif
-
-#ifndef HAVE_NETINET6_IPSEC
-#include <netinet/ipsec.h>
-#else
-#include <netinet6/ipsec.h>
+#ifdef ENABLE_HYBRID
+#include <resolv.h>
 #endif
 
+#include PATH_IPSEC_H
+
 #include "var.h"
 #include "vmbuf.h"
 #include "schedule.h"
@@ -67,12 +68,14 @@
 
 #include "localconf.h"
 #include "remoteconf.h"
+#include "handler.h"
+#include "policy.h"
+#include "proposal.h"
 #include "isakmp_var.h"
 #include "isakmp.h"
 #include "isakmp_inf.h"
 #include "isakmp_quick.h"
 #include "oakley.h"
-#include "handler.h"
 #include "ipsec_doi.h"
 #include "crypto_openssl.h"
 #include "pfkey.h"
@@ -203,9 +206,9 @@ quick_i1send(iph2, msg)
 			"failed to get ID.\n");
 		goto end;
 	}
-	plog(LLV_DEBUG, LOCATION, NULL, "IDci:");
+	plog(LLV_DEBUG, LOCATION, NULL, "IDci:\n");
 	plogdump(LLV_DEBUG, iph2->id->v, iph2->id->l);
-	plog(LLV_DEBUG, LOCATION, NULL, "IDcr:");
+	plog(LLV_DEBUG, LOCATION, NULL, "IDcr:\n");
 	plogdump(LLV_DEBUG, iph2->id_p->v, iph2->id_p->l);
 
 	/*
@@ -725,6 +728,11 @@ quick_i3recv(iph2, msg0)
 			hash = (struct isakmp_pl_hash *)pa->ptr;
 			break;
 		case ISAKMP_NPTYPE_N:
+			if (notify != NULL) {
+				plog(LLV_WARNING, LOCATION, NULL,
+				    "Ignoring multiples notifications\n");
+				break;
+			}
 			isakmp_check_notify(pa->ptr, iph2->ph1);
 			notify = vmalloc(pa->len);
 			if (notify == NULL) {
@@ -984,6 +992,13 @@ quick_r1recv(iph2, msg0)
 			isakmp_check_notify(pa->ptr, iph2->ph1);
 			break;
 
+#ifdef ENABLE_NATT
+		case ISAKMP_NPTYPE_NATOA_DRAFT:
+		case ISAKMP_NPTYPE_NATOA_RFC:
+			/* Ignore original source/destination messages */
+			break;
+#endif
+
 		default:
 			plog(LLV_ERROR, LOCATION, iph2->ph1->remote,
 				"ignore the packet, "
@@ -1053,6 +1068,7 @@ quick_r1recv(iph2, msg0)
 		goto end;
 	}
 
+
 	/* check the existence of ID payload and create responder's proposal */
 	error = get_proposal_r(iph2);
 	switch (error) {
@@ -1583,6 +1599,18 @@ end:
 	return error;
 }
 
+int
+tunnel_mode_prop(p)
+	struct saprop *p;
+{
+	struct saproto *pr;
+
+	for (pr = p->head; pr; pr = pr->next)
+		if (pr->encmode == IPSECDOI_ATTR_ENC_MODE_TUNNEL)
+			return 1;
+	return 0;
+}
+
 /*
  * set SA to kernel.
  */
@@ -1591,7 +1619,6 @@ quick_r3prep(iph2, msg0)
 	struct ph2handle *iph2;
 	vchar_t *msg0;
 {
-	vchar_t *msg = NULL;
 	int error = ISAKMP_INTERNAL_ERROR;
 
 	/* validity check */
@@ -1696,9 +1723,6 @@ quick_r3prep(iph2, msg0)
 	error = 0;
 
 end:
-	if (msg != NULL)
-		vfree(msg);
-
 	return error;
 }
 
@@ -1749,6 +1773,7 @@ quick_ir1mx(iph2, body, hash)
 
 	/* encoding */
 	new = oakley_do_encrypt(iph2->ph1, buf, iph2->ivm->ive, iph2->ivm->iv);
+	
 	if (new == NULL)
 		goto end;
 
@@ -1778,8 +1803,9 @@ get_sainfo_r(iph2)
 	vchar_t *idsrc = NULL, *iddst = NULL;
 	int prefixlen;
 	int error = ISAKMP_INTERNAL_ERROR;
+	int remoteid = 0;
 
-	if (iph2->id_p == NULL) {
+	if (iph2->id == NULL) {
 		switch (iph2->src->sa_family) {
 		case AF_INET:
 			prefixlen = sizeof(struct in_addr) << 3;
@@ -1803,7 +1829,7 @@ get_sainfo_r(iph2)
 		goto end;
 	}
 
-	if (iph2->id == NULL) {
+	if (iph2->id_p == NULL) {
 		switch (iph2->dst->sa_family) {
 		case AF_INET:
 			prefixlen = sizeof(struct in_addr) << 3;
@@ -1827,15 +1853,34 @@ get_sainfo_r(iph2)
 		goto end;
 	}
 
-	iph2->sainfo = getsainfo(idsrc, iddst, iph2->ph1->id_p);
+	{
+		struct remoteconf *conf;
+		conf = getrmconf(iph2->dst);
+		if (conf != NULL)
+			remoteid=conf->ph1id;
+		else{
+			plog(LLV_DEBUG, LOCATION, NULL, "Warning: no valid rmconf !\n");
+			remoteid=0;
+		}
+		
+	}
+
+	iph2->sainfo = getsainfo(idsrc, iddst, iph2->ph1->id_p, remoteid);
 	if (iph2->sainfo == NULL) {
 		plog(LLV_ERROR, LOCATION, NULL,
 			"failed to get sainfo.\n");
 		goto end;
 	}
 
+#ifdef ENABLE_HYBRID
+	/* xauth group inclusion check */
+	if (iph2->sainfo->group != NULL)
+		if(group_check(iph2->ph1,&iph2->sainfo->group->v,1))
+			goto end;
+#endif
+
 	plog(LLV_DEBUG, LOCATION, NULL,
-		"get sa info: %s\n", sainfo2str(iph2->sainfo));
+		"selected sainfo: %s\n", sainfo2str(iph2->sainfo));
 
 	error = 0;
 end:
@@ -1983,7 +2028,17 @@ get_proposal_r(iph2)
 		if (_XIDT(iph2->id_p) == idi2type
 		 && spidx.dst.ss_family == spidx.src.ss_family) {
 			iph2->src_id = dupsaddr((struct sockaddr *)&spidx.dst);
+			if (iph2->src_id  == NULL) {
+				plog(LLV_ERROR, LOCATION, NULL,
+				    "buffer allocation failed.\n");
+				return ISAKMP_INTERNAL_ERROR;
+			}
 			iph2->dst_id = dupsaddr((struct sockaddr *)&spidx.src);
+			if (iph2->dst_id  == NULL) {
+				plog(LLV_ERROR, LOCATION, NULL,
+				    "buffer allocation failed.\n");
+				return ISAKMP_INTERNAL_ERROR;
+			}
 		}
 
 	} else {
@@ -2030,6 +2085,19 @@ get_proposal_r(iph2)
 	if (spidx.ul_proto == 0)
 		spidx.ul_proto = IPSEC_ULPROTO_ANY;
 
+#ifdef HAVE_SECCTX
+	/*
+	 * Need to use security context in spidx to ensure the correct
+	 * policy is selected. The only way to get the security context
+	 * is to look into the proposal sent by peer ahead of time.
+	 */
+	if (get_security_context(iph2->sa, &spidx)) {
+		plog(LLV_ERROR, LOCATION, NULL,
+		     "error occurred trying to get security context.\n");
+		return ISAKMP_INTERNAL_ERROR;
+	}
+#endif /* HAVE_SECCTX */
+
 	/* get inbound policy */
 	sp_in = getsp_r(&spidx);
 	if (sp_in == NULL) {
@@ -2108,5 +2176,12 @@ get_proposal_r(iph2)
 		return ISAKMP_INTERNAL_ERROR;
 	}
 
+#ifdef HAVE_SECCTX
+	if (spidx.sec_ctx.ctx_str) {
+		set_secctx_in_proposal(iph2, spidx);
+	}
+#endif /* HAVE_SECCTX */
+
 	return 0;
 }
+
