@@ -1,4 +1,6 @@
-/* $Id: isakmp_var.h,v 1.9.2.1 2005/05/07 17:26:06 manubsd Exp $ */
+/*	$NetBSD: isakmp_var.h,v 1.6.6.1 2007/02/20 09:08:49 vanhu Exp $	*/
+
+/* Id: isakmp_var.h,v 1.12 2005/05/07 14:45:31 manubsd Exp */
 
 /*
  * Copyright (C) 1995, 1996, 1997, and 1998 WIDE Project.
@@ -120,11 +122,11 @@ extern void log_ph1established __P((const struct ph1handle *));
 
 extern void script_hook __P((struct ph1handle *, int)); 
 extern int script_env_append __P((char ***, int *, char *, char *));
-extern int script_exec __P((int, int, char * const *));
+extern int script_exec __P((char *, int, char * const *));
 
 void purge_remote __P((struct ph1handle *));
-void delete_spd __P((struct ph2handle *));
+void delete_spd __P((struct ph2handle *, u_int64_t));
 #ifdef INET6
 u_int32_t setscopeid __P((struct sockaddr *, struct sockaddr *));
-#endif 
+#endif
 #endif /* _ISAKMP_VAR_H */
