@@ -1,3 +1,5 @@
+/*	$NetBSD: isakmp_xauth.h,v 1.4 2006/09/09 16:22:09 manu Exp $	*/
+
 /*	$KAME$ */
 
 /*
@@ -29,6 +31,9 @@
  * SUCH DAMAGE.
  */
 
+#ifndef _ISAKMP_XAUTH_H
+#define _ISAKMP_XAUTH_H
+
 /* ISAKMP mode config attribute types specific to the Xauth vendor ID */
 #define	XAUTH_TYPE                16520
 #define	XAUTH_USER_NAME           16521
@@ -51,8 +56,9 @@
 #define	XAUTH_STATUS_FAIL       0
 #define	XAUTH_STATUS_OK         1
 
+/* For phase 1 Xauth status */
 struct xauth_state {
-	int status;
+	int status; /* authentication status, used only on server side */
 	int vendorid;
 	int authtype;
 	union {
@@ -61,6 +67,21 @@ struct xauth_state {
 			char *pwd;
 		} generic;
 	} authdata;
+#ifdef HAVE_LIBLDAP
+	char *udn; /* ldap user dn */
+#endif
+};
+
+/* What's been sent */
+#define XAUTH_SENT_USERNAME 1
+#define XAUTH_SENT_PASSWORD 2
+#define XAUTH_SENT_EVERYTHING (XAUTH_SENT_USERNAME | XAUTH_SENT_PASSWORD)
+
+/* For rmconf Xauth data */
+struct xauth_rmconf {
+	vchar_t *login;	/* xauth login */
+	vchar_t *pass;	/* xauth password */
+	int state;      /* what's been sent */
 };
 
 /* status */
@@ -76,21 +97,59 @@ struct xauth_reply_arg {
 };
 
 struct ph1handle;
+struct isakmp_data;
 void xauth_sendreq(struct ph1handle *);
-void xauth_attr_reply(struct ph1handle *, struct isakmp_data *, int);
+int xauth_attr_reply(struct ph1handle *, struct isakmp_data *, int);
 int xauth_login_system(char *, char *);
 void xauth_sendstatus(struct ph1handle *, int, int);
 int xauth_check(struct ph1handle *);
+int group_check(struct ph1handle *, char **, int);
 vchar_t *isakmp_xauth_req(struct ph1handle *, struct isakmp_data *);
 vchar_t *isakmp_xauth_set(struct ph1handle *, struct isakmp_data *);
 void xauth_rmstate(struct xauth_state *);
 void xauth_reply_stub(void *);
-void xauth_reply(struct ph1handle *, int, int, int);
+int xauth_reply(struct ph1handle *, int, int, int);
+int xauth_rmconf_used(struct xauth_rmconf **);
+void xauth_rmconf_delete(struct xauth_rmconf **);
 
 #ifdef HAVE_LIBRADIUS
 int xauth_login_radius(struct ph1handle *, char *, char *);
 int xauth_radius_init(void);
 #endif
+
 #ifdef HAVE_LIBPAM
 int xauth_login_pam(int, struct sockaddr *, char *, char *);
 #endif
+
+#ifdef HAVE_LIBLDAP
+
+#define LDAP_DFLT_HOST		"localhost"
+#define LDAP_DFLT_USER		"cn"
+#define LDAP_DFLT_ADDR		"racoon-address"
+#define LDAP_DFLT_MASK		"racoon-netmask"
+#define LDAP_DFLT_GROUP		"cn"
+#define LDAP_DFLT_MEMBER	"member"
+
+struct xauth_ldap_config {
+	int		pver;
+	vchar_t 	*host;
+	int		port;
+	vchar_t		*base;
+	int		subtree;
+	vchar_t		*bind_dn;
+	vchar_t		*bind_pw;
+	int		auth_type;
+	vchar_t		*attr_user;
+	vchar_t		*attr_addr;
+	vchar_t		*attr_mask;
+	vchar_t		*attr_group;
+	vchar_t		*attr_member;
+};
+
+extern struct xauth_ldap_config xauth_ldap_config;
+
+int xauth_ldap_init(void);
+int xauth_login_ldap(struct ph1handle *, char *, char *);
+#endif
+
+#endif /* _ISAKMP_XAUTH_H */
